@@ -33,27 +33,42 @@ IS_PY3 = version_info[0] == 3
 if IS_PY3:
     unichr = chr
 
-__all__ = ('generate', 'CATEGORIES', 'count', 'parse', 'getone')
+__all__ = ('generate',
+           'CATEGORIES',
+           'count',
+           'parse',
+           'getone',
+           'sre_to_string',
+           'simplify')
 
 CATEGORIES = {'category_space'  : sorted(sre_parse.WHITESPACE)
              ,'category_digit'  : sorted(sre_parse.DIGITS)
              ,'category_any'    : [unichr(x) for x in range(32, 123)]
              }
 
+REVERSE_CATEGORIES = {vv[1]:k for k,v
+                      in sre_parse.CATEGORIES.items() for vv
+                      in v[1]
+                      if v[0] == 'in' and vv[0] == 'category'}
+
+
 def comb(g, i):
     for c in g:
         g2,i = tee(i)
         for c2 in g2:
             yield c+c2
 
+
 def mappend(g, c):
     for cc in g:
         yield cc+c
 
+
 def dappend(g, d, k):
     for cc in g:
         yield cc+d[k]
 
+
 def _in(d):
     ret = []
     neg = False
@@ -104,6 +119,7 @@ def prods(orig, ran, items, limit, grouprefs):
                 for i in ret:
                     yield i
 
+
 def ggen(g1, f, *args, **kwargs):
     groupref = None
     grouprefs = kwargs.get('grouprefs', {})
@@ -118,12 +134,14 @@ def ggen(g1, f, *args, **kwargs):
         else:
             yield g2
 
+
 def concit(g1, seqs, limit, grouprefs):
     for a in g1:
         for s in seqs:
             for b in _gen(s, limit, grouprefs=grouprefs):
                 yield a+b
 
+
 def _gen(d, limit=20, count=False, grouprefs=None):
     """docstring for _gen"""
     if grouprefs == None:
@@ -205,6 +223,7 @@ def _gen(d, limit=20, count=False, grouprefs=None):
 
     return ret
 
+
 def _randone(d, limit=20, grouprefs=None):
     if grouprefs == None:
         grouprefs = {}
@@ -251,6 +270,91 @@ def _randone(d, limit=20, grouprefs=None):
     return ret
 
 
+def sre_to_string(sre_obj, paren=True):
+    """sre_parse object to string
+
+    :param sre_obj: Output of sre_parse.parse()
+    :type sre_obj: list
+    :rtype: str
+    """
+    ret = u''
+    for i in sre_obj:
+        if i[0] == 'in':
+            prefix = ''
+            if len(i[1]) and i[1][0][0] == 'negate':
+                prefix = '^'
+            ret += u'[{0}{1}]'.format(prefix, sre_to_string(i[1], paren=paren))
+        elif i[0] == 'literal':
+            ret += unichr(i[1])
+        elif i[0] == 'category':
+            ret += REVERSE_CATEGORIES[i[1]]
+        elif i[0] == 'any':
+            ret += '.'
+        elif i[0] in 'branch':
+            # TODO simplifications here
+            parts = [sre_to_string(x, paren=paren) for x in  i[1][1]]
+            if not any(parts):
+                continue
+            if i[1][0]:
+                if len(parts) == 1:
+                    paren = False
+                prefix = ''
+            else:
+                prefix = '?:'
+            branch = '|'.join(parts)
+            if paren:
+                ret += '({0}{1})'.format(prefix, branch)
+            else:
+                ret += '{0}'.format(branch)
+        elif i[0] == 'subpattern':
+            if i[1][0]:
+                ret += '({0})'.format(sre_to_string(i[1][1], paren=False))
+            else:
+                ret += '{0}'.format(sre_to_string(i[1][1], paren=paren))
+        elif i[0] == 'not_literal':
+            ret += '[^{0}]'.format(unichr(i[1]))
+        elif i[0] == 'max_repeat':
+            if i[1][0] == i[1][1]:
+                range_str = '{{{0}}}'.format(i[1][0])
+            else:
+                if i[1][0] == 0 and i[1][1] - i[1][0] == sre_parse.MAXREPEAT:
+                    range_str = '*'
+                elif i[1][0] == 1 and i[1][1] - i[1][0] == sre_parse.MAXREPEAT-1:
+                    range_str = '+'
+                else:
+                    range_str = '{{{0},{1}}}'.format(i[1][0], i[1][1])
+            ret += sre_to_string(i[1][2], paren=paren)+range_str
+        elif i[0] == 'groupref':
+            ret += '\\{0}'.format(i[1])
+        elif i[0] == 'at':
+            if i[1] == 'at_beginning':
+                ret += '^'
+            elif i[1] == 'at_end':
+                ret += '$'
+        elif i[0] == 'negate':
+            pass
+            """
+        elif i[0] == 'assert':
+            pass
+        elif i[0] == 'assert_not':
+            pass
+        """
+        else:
+            print('[!] cannot handle expression "%s"' % str(i))
+    return ret
+
+
+def simplify(regex_string):
+    """Simplify a regular expression
+
+    :param regex_string: Regular expression
+    :type regex_string: str
+    :rtype: str
+    """
+    r = parse(regex_string)
+    return sre_to_string(r)
+
+
 def parse(s):
     """Regular expression parser
 
@@ -264,6 +368,7 @@ def parse(s):
         r = sre_parse.parse(s.decode('utf-8'), flags=U)
     return list(r)
 
+
 def generate(s, limit=20):
     """Creates a generator that generates all matching strings to a given regular expression
 
@@ -275,6 +380,7 @@ def generate(s, limit=20):
     """
     return _gen(parse(s), limit)
 
+
 def count(s, limit=20):
     """Counts all matching strings to a given regular expression
 
@@ -287,11 +393,13 @@ def count(s, limit=20):
     """
     return _gen(parse(s), limit, count=True)
 
+
 def getone(regex_string, limit=20):
     """Returns a random matching string to a given regular expression
     """
     return _randone(parse(regex_string), limit)
 
+
 def argparser():
     import argparse
     from sys import stdout
@@ -326,6 +434,11 @@ def argparser():
                      ,default   = False
                      ,action    = 'store_true'
                      )
+    argp.add_argument('-s', '--simplify'
+                     ,help      = 'Simplifies a regular expression'
+                     ,default   = False
+                     ,action    = 'store_true'
+                     )
     argp.add_argument('-d', '--delimiter'
                      ,help      = 'Delimiter - default is \\n'
                      ,default   = '\n'
@@ -341,6 +454,7 @@ def argparser():
                      )
     return vars(argp.parse_args())
 
+
 def __main__():
     from sys import exit, stderr
     args = argparser()
@@ -352,6 +466,9 @@ def __main__():
     if args['random']:
         args['output'].write('%s%s' % (getone(args['regex'], limit=args['limit']), args['delimiter']))
         exit(0)
+    if args['simplify']:
+        args['output'].write('%s%s' % (simplify(args['regex']), args['delimiter']))
+        exit(0)
     try:
         g = generate(args['regex'], args['limit'])
     except Exception as e:
@@ -368,5 +485,6 @@ def __main__():
     if args['delimiter'] == '\n':
         args['output'].write('\n')
 
+
 if __name__ == '__main__':
     __main__()
